@@ -41,7 +41,7 @@
 	trailing l means list (but we also use natural plurals such as "metas")
 	semantic suffixes may be used freely (e.g. e1, e_if, e')
 *)
-
+open Extlib_leftovers
 open Printf
 open Common
 open DisplayTypes.DisplayMode
@@ -158,27 +158,36 @@ let run_command ctx cmd =
 	let t = Timer.timer ["command"] in
 	let cmd = expand_env ~h:(Some h) cmd in
 	let len = String.length cmd in
-	if len > 3 && String.sub cmd 0 3 = "cd " then begin
-		Sys.chdir (String.sub cmd 3 (len - 3));
-		0
-	end else
-	let binary_string s =
-		if not Globals.is_windows then s else String.concat "\n" (Str.split (Str.regexp "\r\n") s)
+	let result =
+		if len > 3 && String.sub cmd 0 3 = "cd " then begin
+			Sys.chdir (String.sub cmd 3 (len - 3));
+			0
+		(* Emit stderr as a server message in server mode *)
+		end else if CompilationServer.runs() then begin
+			let binary_string s =
+				if not Globals.is_windows then s else String.concat "\n" (Str.split (Str.regexp "\r\n") s)
+			in
+			let pout, pin, perr = Unix.open_process_full cmd (Unix.environment()) in
+			let bout = Buffer.create 0 in
+			let berr = Buffer.create 0 in
+			let read_content channel buf =
+				Buffer.add_string buf (IO.read_all (IO.input_channel channel));
+			in
+			let tout = Thread.create (fun() -> read_content pout bout) () in
+			read_content perr berr;
+			Thread.join tout;
+			let result = (match Unix.close_process_full (pout,pin,perr) with Unix.WEXITED c | Unix.WSIGNALED c | Unix.WSTOPPED c -> c) in
+			let serr = binary_string (Buffer.contents berr) in
+			let sout = binary_string (Buffer.contents bout) in
+			if serr <> "" then ctx.messages <- CMError((if serr.[String.length serr - 1] = '\n' then String.sub serr 0 (String.length serr - 1) else serr),null_pos) :: ctx.messages;
+			if sout <> "" then ctx.com.print (sout ^ "\n");
+			result
+		(* Direct pass-through of std streams for normal compilation *)
+		end else begin
+			match Unix.system cmd with
+			| WEXITED c | WSIGNALED c | WSTOPPED c -> c
+		end
 	in
-	let pout, pin, perr = Unix.open_process_full cmd (Unix.environment()) in
-	let bout = Buffer.create 0 in
-	let berr = Buffer.create 0 in
-	let read_content channel buf =
-		Buffer.add_string buf (IO.read_all (IO.input_channel channel));
-	in
-	let tout = Thread.create (fun() -> read_content pout bout) () in
-	read_content perr berr;
-	Thread.join tout;
-	let result = (match Unix.close_process_full (pout,pin,perr) with Unix.WEXITED c | Unix.WSIGNALED c | Unix.WSTOPPED c -> c) in
-	let serr = binary_string (Buffer.contents berr) in
-	let sout = binary_string (Buffer.contents bout) in
-	if serr <> "" then ctx.messages <- CMError((if serr.[String.length serr - 1] = '\n' then String.sub serr 0 (String.length serr - 1) else serr),null_pos) :: ctx.messages;
-	if sout <> "" then ctx.com.print (sout ^ "\n");
 	t();
 	result
 
@@ -558,7 +567,8 @@ let handle_display ctx tctx display_file_dot_path =
 	if ctx.com.display.dms_exit_during_typing then begin
 		if ctx.has_next || ctx.has_error then raise Abort;
 		(* If we didn't find a completion point, load the display file in macro mode. *)
-		ignore(load_display_module_in_macro tctx display_file_dot_path true);
+		if com.display_information.display_module_has_macro_defines then
+			ignore(load_display_module_in_macro tctx display_file_dot_path true);
 		let no_completion_point_found = "No completion point was found" in
 		match com.json_out with
 		| Some _ -> (match ctx.com.display.dms_kind with
@@ -677,7 +687,7 @@ let rec process_params create pl =
 
 and init ctx =
 	let usage = Printf.sprintf
-		"Haxe Compiler %s - (C)2005-2020 Haxe Foundation\nUsage: haxe%s <target> [options] [hxml files...]\n"
+		"Haxe Compiler %s - (C)2005-2020 Haxe Foundation\nUsage: haxe%s <target> [options] [hxml files and dot paths...]\n"
 		(s_version true) (if Sys.os_type = "Win32" then ".exe" else "")
 	in
 	let com = ctx.com in
@@ -715,10 +725,10 @@ try
 	in
 	(* category, official names, deprecated names, arg spec, usage hint, doc *)
 	let basic_args_spec = [
-		("Target",["--js"],["-js"],Arg.String (Initialize.set_platform com Js),"<file>","compile code to JavaScript file");
-		("Target",["--lua"],["-lua"],Arg.String (Initialize.set_platform com Lua),"<file>","compile code to Lua file");
-		("Target",["--swf"],["-swf"],Arg.String (Initialize.set_platform com Flash),"<file>","compile code to Flash SWF file");
-		("Target",["--neko"],["-neko"],Arg.String (Initialize.set_platform com Neko),"<file>","compile code to Neko Binary");
+		("Target",["--js"],["-js"],Arg.String (Initialize.set_platform com Js),"<file>","generate JavaScript code into target file");
+		("Target",["--lua"],["-lua"],Arg.String (Initialize.set_platform com Lua),"<file>","generate Lua code into target file");
+		("Target",["--swf"],["-swf"],Arg.String (Initialize.set_platform com Flash),"<file>","generate Flash SWF bytecode into target file");
+		("Target",["--neko"],["-neko"],Arg.String (Initialize.set_platform com Neko),"<file>","generate Neko bytecode into target file");
 		("Target",["--php"],["-php"],Arg.String (fun dir ->
 			classes := (["php"],"Boot") :: !classes;
 			Initialize.set_platform com Php dir;
@@ -729,7 +739,7 @@ try
 		("Target",["--cppia"],["-cppia"],Arg.String (fun file ->
 			Common.define com Define.Cppia;
 			Initialize.set_platform com Cpp file;
-		),"<file>","generate Cppia code into target file");
+		),"<file>","generate Cppia bytecode into target file");
 		("Target",["--cs"],["-cs"],Arg.String (fun dir ->
 			cp_libs := "hxcs" :: !cp_libs;
 			Initialize.set_platform com Cs dir;
@@ -743,13 +753,13 @@ try
 			Common.define com Define.Jvm;
 			jvm_flag := true;
 			Initialize.set_platform com Java dir;
-		),"<directory>","generate JVM bytecode into target file");
+		),"<file>","generate JVM bytecode into target file");
 		("Target",["--python"],["-python"],Arg.String (fun dir ->
 			Initialize.set_platform com Python dir;
-		),"<file>","generate Python code as target file");
+		),"<file>","generate Python code into target file");
 		("Target",["--hl"],["-hl"],Arg.String (fun file ->
 			Initialize.set_platform com Hl file;
-		),"<file>","compile HL code as target file");
+		),"<file>","generate HashLink .hl bytecode or .c code into target file");
 		("Target",[],["-x"], Arg.String (fun cl ->
 			let cpath = Path.parse_type_path cl in
 			(match com.main_class with
@@ -855,10 +865,10 @@ try
 		),"<file>","use the SWF library for type checking");
 		("Target-specific",["--java-lib"],["-java-lib"],Arg.String (fun file ->
 			add_native_lib file false;
-		),"<file>","add an external JAR or class directory library");
+		),"<file>","add an external JAR or directory of JAR files");
 		("Target-specific",["--java-lib-extern"],[],Arg.String (fun file ->
 			add_native_lib file true;
-		),"<file>","use an external JAR or class directory library for type checking");
+		),"<file>","use an external JAR or directory of JAR files for type checking");
 		("Target-specific",["--net-lib"],["-net-lib"],Arg.String (fun file ->
 			add_native_lib file false;
 		),"<file>[@std]","add an external .NET DLL file");
@@ -946,7 +956,7 @@ try
 		("Compilation",["-C";"--cwd"],[], Arg.String (fun dir ->
 			(* This is handled by process_params, but passed through so we know we did something. *)
 			did_something := true;
-		),"<dir>","set current working directory");
+		),"<directory>","set current working directory");
 		("Compilation",["--haxelib-global"],[], Arg.Unit (fun () -> ()),"","pass --global argument to haxelib");
 	] in
 	let args_callback cl =
@@ -1097,7 +1107,7 @@ with
 			ctx.messages <- [];
 		end else begin
 			error ctx (Printf.sprintf "You cannot access the %s package while %s (for %s)" pack (if pf = "macro" then "in a macro" else "targeting " ^ pf) (s_type_path m) ) p;
-			List.iter (error ctx "    referenced here") (List.rev pl);
+			List.iter (error ctx (Error.compl_msg "referenced here")) (List.rev pl);
 		end
 	| Error.Error (m,p) ->
 		error ctx (Error.error_msg m) p
