@@ -16,7 +16,7 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *)
-
+open Extlib_leftovers
 open Globals
 open JvmGlobals
 open JvmData
@@ -143,6 +143,7 @@ class builder jc name jsig = object(self)
 	val mutable argument_locals = []
 	val mutable thrown_exceptions = Hashtbl.create 0
 	val mutable closure_count = 0
+	val mutable regex_count = 0
 
 	(* per-frame *)
 	val mutable locals = []
@@ -195,6 +196,11 @@ class builder jc name jsig = object(self)
 		closure_count <- closure_count + 1;
 		id
 
+	method get_next_regex_id =
+		let id = regex_count in
+		regex_count <- regex_count + 1;
+		id
+
 	(** Adds the current state of locals and stack as a stack frame. This has to be called on every branch target. **)
 	method add_stack_frame =
 		let locals = self#get_locals_for_stack_frame locals in
@@ -316,15 +322,13 @@ class builder jc name jsig = object(self)
 
 	(** Adds a field named [name] with signature [jsig_field] to the enclosing class, and adds an argument with the same name
 	    to this method. The argument value is loaded and stored into the field immediately. **)
-	method add_argument_and_field (name : string) (jsig_field : jsignature) =
+	method add_argument_and_field (name : string) (jsig_field : jsignature) (flags : FieldAccessFlags.t list) =
 		assert (not (self#has_method_flag MStatic));
-		let jf = new builder jc name jsig_field in
-		jf#add_access_flag 1;
-		jc#add_field jf#export_field;
+		ignore(jc#spawn_field name jsig_field flags);
 		let _,load,_ = self#add_local name jsig_field VarArgument in
 		self#load_this;
 		load();
-		self#putfield jc#get_this_path name jsig_field;
+		self#putfield jc#get_this_path name jsig_field
 
 	(** Constructs a [path] object using the specified construction_kind [kind].
 
@@ -343,7 +347,7 @@ class builder jc name jsig = object(self)
 			code#dup;
 			code#aconst_null haxe_empty_constructor_sig;
 			self#invokespecial path "<init>" (method_sig [haxe_empty_constructor_sig] None);
-			if not no_value then self#set_top_initialized (object_path_sig path);
+			if not no_value then self#replace_top (object_path_sig path);
 			if not no_value then code#dup;
 			let jsigs = f () in
 			self#invokevirtual path "new" (method_sig jsigs None);
@@ -351,7 +355,7 @@ class builder jc name jsig = object(self)
 			if not no_value then code#dup;
 			let jsigs = f () in
 			self#invokespecial path "<init>" (method_sig jsigs None);
-			if not no_value then self#set_top_initialized (object_path_sig path)
+			if not no_value then self#replace_top (object_path_sig path)
 
 	(** Loads the default value corresponding to a given signature. **)
 	method load_default_value = function
@@ -486,8 +490,7 @@ class builder jc name jsig = object(self)
 		in
 		let rec unboxed_to_int () = match code#get_stack#top with
 			| TBool | TByte | TShort | TChar | TInt ->
-				ignore(code#get_stack#pop);
-				code#get_stack#push TInt;
+				self#replace_top TInt;
 			| TLong ->
 				code#l2i;
 			| TFloat ->
@@ -556,7 +559,7 @@ class builder jc name jsig = object(self)
 				self#expect_reference_type
 			end else if is_number_sig name jsig then
 				number_to name
-			else if jsig = object_sig then
+			else if is_dynamic_at_runtime jsig then
 				dynamic_to name
 			else
 				code#checkcast (["java";"lang"],name)
@@ -607,22 +610,20 @@ class builder jc name jsig = object(self)
 			code#l2i;
 			code#i2c;
 		| TBool,TInt ->
-			ignore(code#get_stack#pop);
-			code#get_stack#push TBool;
+			self#replace_top TBool;
 		| TObject(path1,_),TObject(path2,_) when path1 = path2 ->
 			()
 		| TObject((["java";"lang"],"String"),_),_ when allow_to_string ->
 			self#expect_reference_type;
 			self#invokestatic (["haxe";"jvm"],"Jvm") "toString" (method_sig [object_sig] (Some string_sig))
-		| TObject(path1,_),TObject(path2,_) ->
-			if path1 = object_path then begin
+		| TObject(path1,_),t2 ->
+			if is_unboxed t2 then
+				self#expect_reference_type
+			else if path1 = object_path then begin
 				(* We should never need a checkcast to Object, but we should adjust the stack so stack maps are wide enough *)
-				ignore(code#get_stack#pop);
-				code#get_stack#push object_sig
+				self#replace_top object_sig
 			end else
-				code#checkcast path1;
-		| TObject(path,_),TTypeParameter _ ->
-			code#checkcast path
+				code#checkcast path1
 		| TMethod _,TMethod _ ->
 			()
 		| TMethod _,_ ->
@@ -998,9 +999,8 @@ class builder jc name jsig = object(self)
 		in
 		locals <- loop [] locals
 
-	method set_top_initialized jsig =
-		ignore(code#get_stack#pop);
-		code#get_stack#push jsig
+	method replace_top jsig =
+		code#get_stack#replace jsig
 
 	(** This function has to be called once all arguments are declared. *)
 	method finalize_arguments =
