@@ -441,6 +441,11 @@ let emit_string_cca exec1 exec2 p env =
 	if index < 0 || index >= s.slength then vnull
 	else vint (EvalString.char_at s index)
 
+let emit_string_cca_unsafe exec1 exec2 p env =
+	let s = decode_vstring (exec1 env) in
+	let index = decode_int_p (exec2 env) p in
+	vint (EvalString.char_at s index)
+
 (* Write *)
 
 let emit_bytes_length_write exec1 exec2 env =
