@@ -16,7 +16,7 @@
 	along with this program; if not, write to the Free Software
 	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  *)
-
+open Extlib_leftovers
 open Globals
 open EvalHash
 
@@ -93,6 +93,40 @@ type vprototype_kind =
 	| PInstance
 	| PObject
 
+type vhandle =
+	| HLoop of Luv.Loop.t
+	| HIdle of Luv.Idle.t
+	| HTimer of Luv.Timer.t
+	| HAsync of Luv.Async.t
+	| HBuffer of Luv.Buffer.t
+	| HSockAddr of Luv.Sockaddr.t
+	| HTcp of Luv.TCP.t
+	| HUdp of Luv.UDP.t
+	| HPipe of Luv.Pipe.t
+	| HTty of Luv.TTY.t
+	| HFile of Luv.File.t
+	| HDir of Luv.File.Dir.t
+	| HSignal of Luv.Signal.t
+	| HProcess of Luv.Process.t
+	| HRedirection of Luv.Process.redirection
+	| HAddrRequest of Luv.DNS.Addr_info.Request.t
+	| HNameRequest of Luv.DNS.Name_info.Request.t
+	| HFileRequest of Luv.File.Request.t
+	| HRandomRequest of Luv.Random.Request.t
+	| HThreadPoolRequest of Luv.Thread_pool.Request.t
+	| HFileModeNumeric of Luv.File.Mode.numeric
+	| HFsEvent of Luv.FS_event.t
+	| HThread of Luv.Thread.t
+	| HOnce of Luv.Once.t
+	| HMutex of Luv.Mutex.t
+	| HRwLock of Luv.Rwlock.t
+	| HSemaphore of Luv.Semaphore.t
+	| HCondition of Luv.Condition.t
+	| HBarrier of Luv.Barrier.t
+	| HFsPoll of Luv.FS_poll.t
+	| HPrepare of Luv.Prepare.t
+	| HCheck of Luv.Check.t
+
 type value =
 	| VNull
 	| VTrue
@@ -109,6 +143,10 @@ type value =
 	| VFunction of vfunc * bool
 	| VFieldClosure of value * vfunc
 	| VLazy of (unit -> value) ref
+	| VNativeString of string
+	| VHandle of vhandle
+	| VInt64 of Signed.Int64.t
+	| VUInt64 of Unsigned.UInt64.t
 
 and vfunc = value list -> value
 
@@ -203,6 +241,7 @@ and venum_value = {
 and vthread = {
 	mutable tthread : Thread.t;
 	tdeque : vdeque;
+	mutable tevents : value;
 	mutable tstorage : value IntMap.t;
 }
 
@@ -213,13 +252,55 @@ and vdeque = {
 
 and vmutex = {
 	mmutex : Mutex.t;
-	mutable mowner : int option; (* thread ID *)
+	mutable mowner : (int * int) option; (* thread ID * same thread lock count *)
 }
 
 and vlock = {
 	ldeque : vdeque;
 }
 
+let same_handle h1 h2 =
+	match h1, h2 with
+	| HLoop h1, HLoop h2 -> h1 == h2
+	| HIdle h1, HIdle h2 -> h1 == h2
+	| HTimer h1, HTimer h2 -> h1 == h2
+	| HAsync h1, HAsync h2 -> h1 == h2
+	| HBuffer h1, HBuffer h2 -> h1 == h2
+	| HSockAddr h1, HSockAddr h2 -> h1 == h2
+	| HTcp h1, HTcp h2 -> h1 == h2
+	| HPipe h1, HPipe h2 -> h1 == h2
+	| HTty h1, HTty h2 -> h1 == h2
+	| HFile h1, HFile h2 -> h1 == h2
+	| HDir h1, HDir h2 -> h1 == h2
+	| HUdp h1, HUdp h2 -> h1 == h2
+	| HSignal h1, HSignal h2 -> h1 == h2
+	| HProcess h1, HProcess h2 -> h1 == h2
+	| HRedirection h1, HRedirection h2 -> h1 == h2
+	| HFileRequest h1, HFileRequest h2 -> h1 == h2
+	| HNameRequest h1, HNameRequest h2 -> h1 == h2
+	| HAddrRequest h1, HAddrRequest h2 -> h1 == h2
+	| HRandomRequest h1, HRandomRequest h2 -> h1 == h2
+	| HThreadPoolRequest h1, HThreadPoolRequest h2 -> h1 == h2
+	| HFileModeNumeric h1, HFileModeNumeric h2 -> h1 == h2
+	| HFsEvent h1, HFsEvent h2 -> h1 == h2
+	| HThread h1, HThread h2 -> Luv.Thread.equal h1 h2
+	| HOnce h1, HOnce h2 -> h1 == h2
+	| HMutex h1, HMutex h2 -> h1 == h2
+	| HRwLock h1, HRwLock h2 -> h1 == h2
+	| HSemaphore h1, HSemaphore h2 -> h1 == h2
+	| HCondition h1, HCondition h2 -> h1 == h2
+	| HBarrier h1, HBarrier h2 -> h1 == h2
+	| HFsPoll h1, HFsPoll h2 -> h1 == h2
+	| HPrepare h1, HPrepare h2 -> h1 == h2
+	| HCheck h1, HCheck h2 -> h1 == h2
+	| HBuffer _,_ | HAsync _,_ | HTimer _, _ | HLoop _, _ | HIdle _, _ | HSockAddr _, _
+	| HTcp _, _ | HPipe _, _ | HTty _, _ | HFile _, _ | HUdp _, _ | HSignal _, _
+	| HProcess _, _ | HRedirection _, _| HFileRequest _, _ | HAddrRequest _, _
+	| HNameRequest _, _ | HRandomRequest _, _ | HThreadPoolRequest _, _
+	| HFileModeNumeric _, _ | HDir _, _ | HFsEvent _, _ | HThread _, _ | HOnce _, _
+	| HMutex _, _ | HRwLock _, _ | HSemaphore _, _ | HCondition _, _ | HBarrier _, _
+	| HFsPoll _, _ | HPrepare _, _ | HCheck _, _ -> false
+
 let rec equals a b = match a,b with
 	| VTrue,VTrue
 	| VFalse,VFalse
@@ -236,6 +317,8 @@ let rec equals a b = match a,b with
 	| VVector vv1,VVector vv2 -> vv1 == vv2
 	| VFunction(vf1,_),VFunction(vf2,_) -> vf1 == vf2
 	| VPrototype proto1,VPrototype proto2 -> proto1.ppath = proto2.ppath
+	| VNativeString s1,VNativeString s2 -> s1 = s2
+	| VHandle h1,VHandle h2 -> same_handle h1 h2
 	| VLazy f1,_ -> equals (!f1()) b
 	| _,VLazy f2 -> equals a (!f2())
 	| _ -> a == b
@@ -262,6 +345,7 @@ let vint i = VInt32 (Int32.of_int i)
 let vint32 i = VInt32 i
 let vfloat f = VFloat f
 let venum_value e = VEnumValue e
+let vnative_string s = VNativeString s
 
 let s_expr_pretty e = (Type.s_expr_pretty false "" false (Type.s_type (Type.print_context())) e)
 
